(ns wanijo.framework.neo4j
  (:require [neo4j-clj.core :as db]
            [wanijo.framework.devmode :as devmode]
            [clj-time.format :as time-format]
            [clj-time.local :as time-local]
            [clojure.spec.alpha :as spec]
            [clojure.string :as cljs])
  (:import (java.util UUID)))

(spec/def ::date-str
  #(re-matches #"\d{4}\d{2}\d{2}T\d{2}\d{2}\d{2}\.\d{3}Z" %))

(spec/def ::uuid
  #(re-matches
    #"[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}" %))

(def conn
  (delay (db/connect
          "bolt://localhost:7687"
          "neo4j"
          "b")))

(defmacro defquery [& args] `(db/defquery ~@args))

(defn uuid []
  (str (UUID/randomUUID)))

(defn butiful-query [qry]
  (->> qry
       str
       cljs/trim-newline
       cljs/split-lines
       (map cljs/trim)
       (filter #(pos? (count %)))
       (cljs/join \newline)))

(defn exec-query! [qry params]
  (with-open [session (db/get-session @conn)]
    (devmode/send-to-bar
     (str (butiful-query qry)
          "<br>---Params---<br>"
          params))
    (qry session params)))

(defn exec-queries! [& tuples]
  (db/with-transaction @conn tx
    (doseq [tuple tuples]
      (let [qry (first tuple)
            params (second tuple)]
        (devmode/send-to-bar
         (str (butiful-query qry)
              "<br>---Params---<br>"
              params))
        (qry tx params)))))

(defn now-str []
  (time-format/unparse
   (time-format/formatters :basic-date-time)
   (time-local/local-now)))
